#!/usr/bin/env python
"""
FUNCTION: Adds the full taxonomy strings of the species of the HMM hit
          proteins to the database. For this it read a 'tip taxonomy file'
          produced by build_NCBI_tip_taxonomies.py.

USAGE: NHase_db_add_species_properties.py <DB_FILE> <TAXONOMY_STRING_FILE> <DB_SWITCH>

       <DB_SWITCH> can be
         - 'insert' to put new data into the database
         - 'update' to change entries in the database

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys
import nhase_db_tools

def main():
    if not len(sys.argv) == 4  or (not sys.argv[3] in ['insert', 'update']):
        print __doc__
        sys.exit(2)
    adder = SpeciesPropertyAdder(sys.argv[1])
    adder.build_list_of_species_in_db()
    adder.read_taxonomy_file(sys.argv[2], sys.argv[3])

class SpeciesPropertyAdder(object):

    def __init__(self, db_file):
        self.db_connector = nhase_db_tools.NHasesDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
            
    def build_list_of_species_in_db(self):
        self.species_list = []
        self.db_connector.create_db_connection()
        cur = self.db_connector.con.cursor()
        cur.execute('SELECT DISTINCT species FROM proteins;')
        for species in cur.fetchall():
            # Skip uncultured organism as they are maybe place to
            # wrong taxonomies
            if species[0].startswith('uncultured'): continue
            self.species_list.append(species[0])
        self.db_connector.close_db_connection()

    def read_taxonomy_file(self, input_file, insert_update_switch):
        for line in open(input_file):
            if line[0] in ['#', '\n']: continue
            if self._check_for_odd_lines(line) == True: continue
            found_switch = False
            for taxonomy_level in line.split('|'):
                if found_switch == True: break
                for species in self.species_list:
                    if taxonomy_level.startswith(species):
                        taxonomy_string = line[:-1]
                        taxonomy_string = taxonomy_string.replace('"', '')
                        taxonomy_string = taxonomy_string.replace("'", '')
                        if insert_update_switch == 'insert':
                            self.db_connector.fill_species_table(
                                species, taxonomy_string)
                        elif insert_update_switch == 'update':
                            self.db_connector.update_species_table(
                                species, taxonomy_string)
                        found_switch = True
                        self.species_list.remove(species)
                        break

    def _check_for_odd_lines(self, line):
        """Seaches for oddities in the taxonomy strings"""
        if \
            line.startswith('root|other sequences') or \
            line.startswith('root|Viruses') or \
            line.startswith('root|biota|Eucarya|unclassified eukaryotes'):
            return(True)
        return(False)

if __name__ == '__main__': main()
