#!/usr/bin/env python
"""
FUNCTION: Analyses the protein annotation strings and tries to put the
          protein into a group based on keywords.

USAGE: NHase_db_analyse_annotation_strings.py <DB_FILE>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import nhase_db_tools

def main():
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)
    analyser = AnnotationStringAnalyser(sys.argv[1])
    analyser.analyse_annotation_strings()

class AnnotationStringAnalyser(object):

    def __init__(self, db_file):
        self.db_connector = nhase_db_tools.NHasesDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self._set_search_word_lists()

    def analyse_annotation_strings(self):
        self.db_connector.create_db_connection()
        cur = self.db_connector.con.cursor()
        cur.execute('SELECT protein_name, annotation FROM proteins;')
        for protein_name, annotation in cur.fetchall():
            annotation_group = self._search_for_nhase_keywords(annotation)
            if annotation_group == 'Unknown':
                if self._search_string_set(
                    self.thiocyanate_hydrolase_names, annotation) == True:
                    annotation_group = 'Thiocyanate_hydrolase'
            self.db_connector.fill_protein_annotation_table(
                protein_name, annotation_group)
        self.db_connector.close_db_connection()

    def _search_for_nhase_keywords(self, annotation):
        annotation_index = 0
        nhase_words = ['Nitrile hydratase', 'nitrile hydratase',
                       'NITRILE HYDRATASE']
        weakenizers = ['Hypothetical', 'hypothetical', 'HYPOTHETICAL',
                       'Putative', 'putative', 'Probable', 'probable',
                       'similar', 'Similar', 'PREDICTED']
        for nhase_word in nhase_words:
            if annotation_index < 1  and nhase_word in annotation:
                annotation_index = 1
        for weakenizer in weakenizers:
            if weakenizer in annotation and (
               annotation_index == 1 or
               annotation_index == -1):
                annotation_index *= 0.5
        return(self._annotation_index_to_group_name(annotation_index))

    def _search_string_set(self, string_set, annotation):
        for string in string_set:
            if string in annotation:
                return(True)
        return(False)

    def _annotation_index_to_group_name(self, annotation_index):
        if annotation_index == 1:
            return('NHase')
        elif annotation_index == 0.5:
            return('Putative_NHase')
        elif annotation_index == 0:
            return('Unknown')

    def _set_search_word_lists(self):
        self.thiocyanate_hydrolase_names = ['Thiocyanate hydrolase', 'ScnC']

if __name__ == '__main__': main()
