#!/usr/bin/env python
"""
FUNCTION: Searches for protein with more than one HMM hit.

USAGE: NHase_db_find_domain_fusion.py <DB_FILE>


Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import nhase_db_tools

def main():
    
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)
        
    finder = FusionFinder(sys.argv[1])
    finder.find_fusion_proteins()

class FusionFinder(object):

    def __init__(self, db_file):
        self.db_connector = nhase_db_tools.NHasesDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()

    def find_fusion_proteins(self):
        single_hit_protein_counter = 0
        multi_hit_proteins = []
        for protein in self.db_connector.get_all_used_proteins():
            hits = self.db_connector.get_all_hits_of_protein(protein)
            if len(hits) == 1: single_hit_protein_counter += 1
            else:
                multi_hit_proteins.append([protein, hits])
        print 'Number of single hit proteins: %s' % (single_hit_protein_counter)
        print 'Number of multi hit proteins: %s' % len(multi_hit_proteins)
        print 'The following proteins have more than one hit:'
        for protein in multi_hit_proteins:
            print '* %s has the following hits' % protein[0]
            for hit in protein[1]:
                print ' - %s' % (hit)

if __name__ == '__main__': main()
