"""
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import os
try:
    # For running on OpenBSD
    from pysqlite2 import dbapi2 as sqlite
except:
    # For running on openSUSE
    import sqlite

class NHasesDBConnector(object):

    def __init__(self, db_file):
        self.set_db_file(db_file)

    def set_db_file(self, db_file):
        self.db_file = db_file

    def create_db_connection(self):
        self.con = sqlite.connect(self.db_file, isolation_level=None)

    def get_connection(self):
        return(self.con)

    def execute_add_command(self, command):
        try:
            self.con.execute(command)
        except sqlite.IntegrityError:
            print 'The primary key int command "%s" exists already.' % (command)
        except sqlite.OperationalError:
            print 'This command caused serious trouble:\n"%s"' % (command)
        self.con.commit()
        
    def close_db_connection(self):
        pass

    def check_db_file(self):
        return(os.path.exists(self.db_file))

    def fill_hit_table(self, hit_features, hit_id, added_by_cluster_of):
        self.execute_add_command(
            'INSERT INTO HMM_hits '
            '(domain_HMM_hit_name, HMM_hit_name, HMM_domain_name, '
            'hit_protein_name, hit_length, hit_start_position, '
            'hit_end_position, e_value, bitscore, simplified_hit_header, '
            'added_by_cluster_of) '
            'VALUES ("%s_%s", "%s", "%s", "%s", "%s", "%s", "%s", "%s", '
            '"%s", "%s", "%s");' % (
            hit_features['HMM_domain_name'],
            hit_features['hit_name'],
            hit_features['hit_name'],
            hit_features['HMM_domain_name'],
            hit_features['hit_protein_name'],
            hit_features['hit_length'],
            hit_features['hit_start_position'],
            hit_features['hit_end_position'],
            hit_features['e_value'],
            hit_features['bitscore'],
            hit_id,
            added_by_cluster_of))

    def update_hit_table(self, hit_features, hit_id, added_by_cluster_of):
        self.execute_add_command(
            'UPDATE HMM_hits SET domain_HMM_hit_name="%s_%s", HMM_hit_name="%s", '
            'HMM_domain_name="%s", hit_protein_name="%s", hit_length="%s", '
            'hit_start_position="%s", hit_end_position="%s", e_value="%s", '
            'bitscore="%s", simplified_hit_header="%s", '
            'added_by_cluster_of="%s";' % (
            hit_features['HMM_domain_name'],
            hit_features['hit_name'],
            hit_features['hit_name'],
            hit_features['HMM_domain_name'],
            hit_features['hit_protein_name'],
            hit_features['hit_length'],
            hit_features['hit_start_position'],
            hit_features['hit_end_position'],
            hit_features['e_value'],
            hit_features['bitscore'],
            hit_id,
            added_by_cluster_of))

    def fill_protein_table(self, protein_features):
        self.execute_add_command(
            'INSERT INTO proteins (protein_name, length, annotation, '
            'species, source) VALUES ("%s", "%s", "%s", "%s", "%s");' % (
            protein_features['protein_name'],
            protein_features['protein_length'],
            protein_features['annotation'],
            protein_features['species'],
            protein_features['source']))

    def update_protein_table(self, protein_features):
        self.execute_add_command(
            'UPDATE proteins SET protein_name="%s", length="%s", '
            'annotation="%s", species="%s", source="%s";'% (
            protein_features['protein_name'],
            protein_features['protein_length'],
            protein_features['annotation'],
            protein_features['species'],
            protein_features['source']))

    def fill_species_table(self, species_name, taxonomy_string):
        self.execute_add_command(
            'INSERT INTO species (species_name, full_taxonomie_string '
            ') VALUES ("%s", "%s");' % (species_name, taxonomy_string))

    def update_species_table(self, species_name, taxonomy_string):
        self.execute_add_command(
            'UPDATE species SET full_taxonomie_string="%s" '
            'WHERE species_name="%s";' % (taxonomy_string, species_name))

    def fill_tree_position_table(self, domain_HMM_hit_name, tree_position):
        self.execute_add_command(
            'INSERT INTO hit_tree_position (domain_HMM_hit_name, '
            ' tree_position) VALUES ("%s", "%s");' % (
            domain_HMM_hit_name, tree_position))

    def update_tree_position_table(self, domain_HMM_hit_name, tree_position):
        self.execute_add_command(
            'UPDATE hit_tree_position SET tree_position="%s" WHERE '
            'domain_HMM_hit_name="%s";' % (tree_position, domain_HMM_hit_name))

    def fill_protein_annotation_table(self, protein_name, annotation_group):
        self.execute_add_command(
            'INSERT INTO protein_annotations (protein_name, '
            'text_string_annotation_group ) VALUES ("%s", "%s");' % (
            protein_name, annotation_group))

    def fill_tree_based_hit_annotations_table(
        self, domain_HMM_hit_name, annotation_group, comments):
        self.execute_add_command(
            'INSERT INTO tree_based_hit_annotations (domain_HMM_hit_name, '
            'tree_based_annotation_group, comments) VALUES ("%s", "%s", "%s");' % (
            domain_HMM_hit_name, annotation_group, comments))

    def update_tree_based_hit_annotations_table(
        self, domain_HMM_hit_name, annotation_group, comments):
        self.execute_add_command(
            'UPDATE tree_based_hit_annotations SET '
            'tree_based_annotation_group="%s", comments="%s" '
            'WHERE domain_HMM_hit_name="%s";' % (
             annotation_group, comments, domain_HMM_hit_name))
        
    def create_db_file(self):
        print 'The database file "%s" does not exist and will be created now.' \
              % self.db_file
        self.create_db_connection()
        self.con.execute(
            'CREATE TABLE HMM_hits ('
            'domain_HMM_hit_name text primary key, '
            'HMM_hit_name text, '
            'HMM_domain_name text, '
            'simplified_hit_header text, '
            'hit_protein_name text, '
            'hit_length int, '
            'hit_start_position int, '
            'hit_end_position int, '
            'e_value text, '
            'bitscore text, '
            'added_by_cluster_of)')
        self.con.execute(
            'CREATE TABLE proteins ('
            'protein_name text primary key, '
            'length int, '
            'annotation text, '
            'species text, '
            'source text)')
        self.con.execute(
            'CREATE TABLE species ('
            'species_name text primary key, '
            'full_taxonomie_string text)')
        self.con.execute(
            'CREATE TABLE hit_tree_position ('
            'domain_HMM_hit_name text primary key, '
            'tree_position int)')
        self.con.execute(
            'CREATE TABLE protein_annotations ('
            'protein_name text primary key, '
            'text_string_annotation_group text)')
        self.con.execute(
            'CREATE TABLE tree_based_hit_annotations ('
            'domain_HMM_hit_name text primary key, '
            'tree_based_annotation_group text, '
            'comments text)')
        self.close_db_connection()

    def get_protein_annotation(self, hit_protein_name):
        annotation_group = '-'
        cur = self.con.cursor()
        cur.execute('SELECT text_string_annotation_group FROM '
                    'protein_annotations WHERE protein_name="%s";' % (
            hit_protein_name))
        row = cur.fetchone()
        cur.close()
        if not row == None:
            annotation_group = row[0]
        return(annotation_group)

    def get_all_used_hits(self):
        hits = []
        cur = self.con.cursor()
        cur.execute('SELECT DISTINCT domain_HMM_hit_name FROM HMM_hits;')
        for hit in cur.fetchall():
            hits.append(hit[0])
        return(hits)

    def get_all_used_domains(self):
        domains = []
        cur = self.con.cursor()
        cur.execute('SELECT DISTINCT HMM_domain_name FROM HMM_hits;')
        for domain in cur.fetchall():
            domains.append(domain[0])
        return(domains)

    def get_all_used_proteins(self):
        proteins = []
        cur = self.con.cursor()
        cur.execute('SELECT DISTINCT protein_name FROM proteins;')
        for protein in cur.fetchall():
            proteins.append(protein[0])
        return(proteins)

    def get_all_used_species(self):
        species = []
        cur = self.con.cursor()
        cur.execute('SELECT DISTINCT species FROM proteins;')
        for a_species in cur.fetchall():
            species.append(a_species[0])
        return(species)

    def get_all_used_sources(self):
        sources = []
        cur = self.con.cursor()
        cur.execute('SELECT DISTINCT source FROM proteins;')
        for source in cur.fetchall():
            sources.append(source[0])
        return(sources)

    def get_all_hits_of_domain(self, HMM_domain_name):
        hits = []
        cur = self.con.cursor()
        cur.execute('SELECT domain_HMM_hit_name FROM HMM_hits '
                    'WHERE HMM_domain_name="%s";' % (HMM_domain_name))
        for domain_HMM_hit_name in cur.fetchall():
            hits.append(domain_HMM_hit_name[0])
        return(hits)

    def get_all_proteins_of_source(self, source):
        proteins = []
        cur = self.con.cursor()
        cur.execute('SELECT DISTINCT protein_name FROM proteins '
                    'WHERE source="%s";' % (source))
        for protein in cur.fetchall():
            proteins.append(protein[0])
        return(proteins)

    def get_all_proteins_of_species(self, species):
        proteins = []
        cur = self.con.cursor()
        cur.execute('SELECT protein_name FROM proteins WHERE species="%s";' % (
            species))
        for protein in cur.fetchall():
            proteins.append(protein[0])
        return(proteins)

    def get_protein_of_a_hit(self, domain_HMM_hit_name):
        cur = self.con.cursor()
        cur.execute('SELECT hit_protein_name FROM HMM_hits '
                    'WHERE domain_HMM_hit_name="%s";' % (domain_HMM_hit_name))
        potential_protein = cur.fetchone()
        if potential_protein == None:
            return(None)
        else:
            return(potential_protein[0])

    def get_annotation_string_of_protein(self, protein_name):
        cur = self.con.cursor()
        cur.execute('SELECT annotation FROM proteins '
                    'WHERE protein_name="%s";' % (protein_name))
        potential_annotation = cur.fetchone()
        if potential_annotation == None:
            return(None)
        else:
            return(potential_annotation[0])

    def get_length_of_protein(self, protein_name):
        cur = self.con.cursor()
        cur.execute('SELECT length FROM proteins '
                    'WHERE protein_name="%s";' % (protein_name))
        potential_length = cur.fetchone()
        if potential_length == None:
            return(None)
        else:
            return(potential_length[0])

    def get_all_hits_of_protein(self, protein_name):
        hits = []
        cur = self.con.cursor()
        cur.execute('SELECT domain_HMM_hit_name FROM HMM_hits '
                    'WHERE hit_protein_name="%s";' % (protein_name))
        for hit in cur.fetchall():
            hits.append(hit[0])
        return(hits)

    def get_tree_based_annotation_of_hit(self, domain_HMM_hit_name):
        cur = self.con.cursor()
        cur.execute('SELECT tree_based_annotation_group FROM '
                    'tree_based_hit_annotations WHERE '
                    'domain_HMM_hit_name="%s";' % (
            domain_HMM_hit_name))
        potential_tree_based_annotation = cur.fetchone()
        if potential_tree_based_annotation == None:
            return(None)
        else:
            return(potential_tree_based_annotation[0])

    def get_source_of_hit(self, domain_HMM_hit_name):
        hit_protein = self.get_protein_of_a_hit(domain_HMM_hit_name)
        if hit_protein == None: return(None)
        source = self.get_source_of_protein(hit_protein)
        return(source)

    def get_source_of_protein(self, protein_name):
        cur = self.con.cursor()
        cur.execute('SELECT source FROM proteins WHERE '
                    'protein_name="%s";' % (protein_name))
        potential_source = cur.fetchone()
        if potential_source == None:
            return(None)
        else:
            return(potential_source[0])

    def get_species_of_protein(self, protein_name):
        cur = self.con.cursor()
        cur.execute('SELECT species FROM proteins WHERE '
                    'protein_name="%s";' % (protein_name))
        potential_species = cur.fetchone()
        if potential_species == None:
            return(None)
        else:
            return(potential_species[0])

    def get_all_hits_with_the_given_tree_anno(
        self, tree_based_annotation_group):
        hits = []
        cur = self.con.cursor()
        cur.execute('SELECT domain_HMM_hit_name FROM '
                    'tree_based_hit_annotations '
                    'WHERE tree_based_annotation_group="%s";' % (
            tree_based_annotation_group))
        for hit in cur.fetchall():
            hits.append(hit[0])
        return(hits)

    def get_full_tax_of_species(self, species_name):
        cur = self.con.cursor()
        cur.execute('SELECT full_taxonomie_string FROM species WHERE '
                    'species_name="%s";' % (species_name))
        potential_tax_string = cur.fetchone()
        if potential_tax_string == None:
            return(None)
        else:
            return(potential_tax_string[0])

    def get_text_string_annotation_group_of_protein(self, protein_name):
        cur = self.con.cursor()
        cur.execute('SELECT text_string_annotation_group FROM '
                    'protein_annotations WHERE protein_name="%s";' % (
            protein_name))
        potential_annotataion_group = cur.fetchone()
        if potential_annotataion_group == None:
            return(None)
        else:
            return(potential_annotataion_group[0])

    def build_domain_composition_string(self, protein_name):
        domain_composition_string = ''
        start_positions_and_hits = []
        for domain_HMM_hit_name in self.get_all_hits_of_protein(protein_name):
            hit_features = self.get_hit_features(domain_HMM_hit_name)
            start_positions_and_hits.append(
                [hit_features['hit_start_position'], domain_HMM_hit_name,
                 hit_features['HMM_domain_name']])
        start_positions_and_hits.sort()
        for start_position, domain_HMM_hit_name, HMM_domain_name in \
                start_positions_and_hits:
            tree_based_annotation_group = self.get_tree_based_annotation_of_hit(
                domain_HMM_hit_name)
            domain_composition_string += '%s(%s):' % (
                HMM_domain_name, tree_based_annotation_group)
        domain_composition_string = domain_composition_string[:-1]
        return(domain_composition_string)

    def get_hit_features(self, domain_HMM_hit_name):
        hit_features = {}
        cur = self.con.cursor()
        cur.execute('SELECT simplified_hit_header, hit_protein_name, '
                    'hit_length, hit_start_position, hit_end_position, '
                    'e_value, bitscore, added_by_cluster_of, HMM_domain_name '
                    'FROM HMM_hits WHERE domain_HMM_hit_name="%s";' % (
            domain_HMM_hit_name))
        (hit_features['simplified_hit_header'],
         hit_features['hit_protein_name'],
         hit_features['hit_length'],
         hit_features['hit_start_position'],
         hit_features['hit_end_position'],
         hit_features['e_value'],
         hit_features['bitscore'],
         hit_features['added_by_cluster_of'],
         hit_features['HMM_domain_name']) = cur.fetchone()
        cur.close()
        return(hit_features)

class HMMFastaToolBox(object):

    def get_hit_features(self, full_header):
        hit_name = full_header.split(' ')[0]
        bitscore = '-'
        e_value = '-'
        HMM_domain_name = '-'
        hit_length = '0'
        hit_start_position = '0'
        hit_end_position = '0'

        # For usual hits
        if '_from_' in hit_name and not hit_name.endswith('.seq'):
            hit_protein_name = full_header.split(
                'in sequence "')[1].split('", ')[0]
            bitscore = float(
                full_header.split(', bitscore: ')[1].split(', e-value:')[0])
            e_value = float(
                full_header.split(', e-value: ')[1].split(', length:')[0])
            hit_length = int(
                full_header.split(', length: ')[1].split(')')[0])
            HMM_domain_name = full_header.split(
                '(HMM "')[1].split('" in')[0].replace('PKS_', '')
            hit_start_position, hit_end_position = self.__get_hit_coordinates(
                hit_name)
        
        return({'hit_name' : hit_name,
                'hit_protein_name' : hit_protein_name,
                'bitscore' : bitscore,
                'e_value' : e_value,
                'hit_length' : hit_length,
                'HMM_domain_name' : HMM_domain_name,
                'hit_start_position' : hit_start_position,
                'hit_end_position' : hit_end_position})

    def __get_hit_coordinates(self, hit_name):
        hit_start = int(hit_name.split('_from_')[1].split('_to_')[0])
        hit_end = int(hit_name.split('_to_')[1])
        return (hit_start, hit_end)
